#!/usr/bin/env python3
# Authors: Edificom SA <dev-auto@edificom.ch>; martinmartossimon@gmail.com
#
# Monitor status of Fortinet SLAs
#
# sample snmpwalk:
# .1.3.6.1.4.1.12356.101.4.9.2.1.14.X 	CL-AWS-01    		LinkIfName (Str)
# .1.3.6.1.4.1.12356.101.4.9.2.1.4.X 	[0, 1]    		LinkState (int)
# .1.3.6.1.4.1.12356.101.4.9.2.1.9.X 	1.000 - 100.000 	LinkPacketLoss (Percent)
# .1.3.6.1.4.1.12356.101.4.9.2.1.5.X    0.000 - 123.251		LinkLatency (ms)

import datetime
from dataclasses import dataclass
from typing import List

from cmk.base.plugins.agent_based.agent_based_api.v1 import (  # check_levels,
    Metric,
    Result,
    Service,
    SNMPTree,
    State,
    exists,
    register,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import StringTable

# from cmk.base.check_api import check_levels


@dataclass
class sla_fortinet:
    LinkIfName: str
    LinkState: int
    LinkPacketLoss: float
    LinkLatency: float
    LinkJitter: float
    LinkName: str
    LinkVdom: str


def parse_sla(string_table: StringTable) -> List[sla_fortinet]:
    listLinkInfos = []
    for linkInfo in string_table:
        (
            LinkIfName,
            LinkState,
            LinkLatency,
            LinkPacketLoss,
            LinkJitter,
            LinkName,
            LinkVdom,
        ) = linkInfo
        listLinkInfos.append(
            sla_fortinet(
                LinkIfName=LinkIfName,
                LinkState=int(LinkState),
                LinkPacketLoss=float(LinkPacketLoss),
                LinkLatency=float(LinkLatency),
                LinkJitter=float(LinkJitter),
                LinkName=str(LinkName),
                LinkVdom=str(LinkVdom),
            )
        )
    return listLinkInfos


def discovery_sla_fortinet(section):
    for service in section:
        yield Service(
            item=service.LinkIfName + " " + service.LinkName,
            parameters={
                "warning_latency": 100,
                "critical_latency": 200,
                "warning_loss": 5,
                "critical_loss": 20,
                "warning_jitter": 20,
                "critical_jitter": 30,
            },
        )


def check_sla_fortinet(item, params, section):
    now = datetime.datetime.now()
    if section:
        for item2 in section:
            if item == item2.LinkIfName + " " + item2.LinkName:
                latencyWarning = params.get("warning_latency", None)
                latencyCritical = params.get("critical_latency", None)
                lossWarning = params.get("warning_loss", None)
                lossCritical = params.get("critical_loss", None)
                jitterWarning = params.get("warning_jitter", None)
                jitterCritical = params.get("critical_jitter", None)
                if item2.LinkState == 0:
                    yield Metric("State", int(item2.LinkState))
                    yield Metric(
                        "Latency",
                        float(item2.LinkLatency),
                        levels=(latencyWarning, latencyCritical),
                    )
                    yield Metric(
                        "PacketLoss",
                        float(item2.LinkPacketLoss),
                        levels=(lossWarning, lossCritical),
                    )
                    yield Metric(
                        "Jitter",
                        float(item2.LinkJitter),
                        levels=(jitterWarning, jitterCritical),
                    )

                    yield Result(
                        state=State.OK,
                        summary="State: {}".format(
                            item2.LinkState,
                        ),
                    )

                    if item2.LinkLatency >= latencyCritical:
                        status = State.CRIT
                    elif item2.LinkLatency >= latencyWarning:
                        status = State.WARN
                    else:
                        status = State.OK
                    yield Result(
                        state=status,
                        summary="Latency: {}".format(
                            item2.LinkLatency,
                        ),
                    )

                    if item2.LinkPacketLoss >= lossCritical:
                        status = State.CRIT
                    elif item2.LinkPacketLoss >= lossWarning:
                        status = State.WARN
                    else:
                        status = State.OK
                    yield Result(
                        state=status,
                        summary="PacketLoss: {}".format(
                            item2.LinkPacketLoss,
                        ),
                    )

                    if item2.LinkJitter >= jitterCritical:
                        status = State.CRIT
                    elif item2.LinkJitter >= jitterWarning:
                        status = State.WARN
                    else:
                        status = State.OK
                    yield Result(
                        state=status,
                        summary="Jitter: {}".format(
                            item2.LinkJitter,
                        ),
                    )
                    # check_levels

    else:
        yield Result(
            state=State.UNKNOWN,
            summary="No metrics collected on last check: %s" % (now),
        )


register.snmp_section(
    name="fortigate_sla",
    detect=exists(".1.3.6.1.4.1.12356.101.4.9.2.1.*"),
    # detect = startswith(".1.3.6.1.2.1.47.1.1.1.1.2.1", "fortinet"),
    fetch=SNMPTree(
        base=".1.3.6.1.4.1.12356.101.4.9.2.1",
        oids=[
            "14",  # LinkIfName
            "4",  # LinkState
            "5",  # LinkLatency
            "9",  # LinkPacketLoss
            "6",  # LinkJitter
            "2",  # LinkName
            "10",  # LinkVdom
            # "1",  # LinkID
            # "3",  # LinkSeq
            # "11",  # LinkBandwidthIn
            # "12",  # LinkBandwidthOut
            # "13",  # LinkBandwidthBi
        ],
    ),
    parse_function=parse_sla,
)

register.check_plugin(
    name="fortigate_sla",
    service_name="SLA %s",
    discovery_function=discovery_sla_fortinet,
    check_function=check_sla_fortinet,
    check_ruleset_name="fortigate_sla",
    check_default_parameters={
        "warning_latency": 100,
        "critical_latency": 200,
        "warning_loss": 5,
        "critical_loss": 20,
        "warning_jitter": 20,
        "critical_jitter": 30,
    },
)
